@@ -81,7 +81,8 @@ def _create_empty_datashelf(pathToDataself,
         
 def create_personal_setting(modulePath, OS):
     
-    
+    from . import config 
+    os.makedirs(config.CONFIG_DIR, exist_ok=True)
     # Linux 
     if OS == 'Linux':
         import tkinter as tk
@@ -112,8 +113,8 @@ def create_personal_setting(modulePath, OS):
         file_path_variable = input("Please enter path to datashelf: ")
     
     fin = open(os.path.join(modulePath, 'data','personal_template.py'), 'r')
-    os.makedirs(os.path.join(modulePath, 'settings'),exist_ok=True)
-    fout = open(os.path.join(modulePath, 'settings','personal.py'), 'w')
+    
+    fout = open(os.path.join(config.CONFIG_DIR, 'personal.py'), 'w')
     
     for line in fin.readlines():
         outLine = line.replace('XX',userName).replace('/PPP/PPP', file_path_variable)
@@ -122,20 +123,24 @@ def create_personal_setting(modulePath, OS):
     fout.close()
 
 
-def create_initial_config(modulePath, 
+def create_initial_config(module_path,
+                          config_path, 
                           write_config=True):
     import git
-    fin = open(os.path.join(modulePath, 'data','personal_template.py'), 'r')
-    os.makedirs(os.path.join(modulePath, 'settings'),exist_ok=True)
+    fin = open(os.path.join(module_path, 'data','personal_template.py'), 'r')
+    
     
     
     DEBUG = False
     READ_ONLY = True
-    sandboxPath = os.path.join(modulePath, 'data','SANDBOX_datashelf')
+    sandboxPath = os.path.join(module_path, 'data','SANDBOX_datashelf')
+    
     git.Repo.init(sandboxPath)
     
     if write_config:
-        fout = open(os.path.join(modulePath, 'settings','personal.py'), 'w')
+        #create directory
+        os.makedirs(config_path, exist_ok=True)
+        fout = open(os.path.join(config_path, 'personal.py'), 'w')
         for line in fin.readlines():
             outLine = line.replace('/PPP/PPP', sandboxPath)
             fout.write(outLine)
@@ -199,7 +204,7 @@ def _re_link_functions(dt):
     dt.removeTable = dt.core.DB.removeTable
     dt.removeTables = dt.core.DB.removeTables
     
-    dt.find         = dt.core.DB.getInventory
+    dt.findc        = dt.core.DB.findc
     dt.findp        = dt.core.DB.findp
     dt.findExact    = dt.core.DB.findExact
     dt.getTable     = dt.core.DB.getTable
@@ -231,11 +236,10 @@ def set_autoload_source(boolean):
     from . import config
     config.AUTOLOAD_SOURCES = boolean
     
-    fin = open(os.path.join(config.MODULE_PATH, 'settings','personal.py'), 'r')
+    fin = open(os.path.join(config.CONFIG_DIR, 'personal.py'), 'r')
     lines = fin.readlines()
     fin.close()
-    # os.makedirs(os.path.join(config.MODULE_PATH, 'settings'),exist_ok=True)
-    fout = open(os.path.join(config.MODULE_PATH, 'settings','personal.py'), 'w')
+    fout = open(os.path.join(config.CONFIG_DIR, 'personal.py'), 'w')
     
     line_found = False
     for line in lines:
@@ -258,15 +262,57 @@ def switch_database_to_testing():
     from . import config
 #    from datatoolbox.tools.install_support import create_initial_config
 #    
-    _, sandboxPath, READ_ONLY, DEBUG = create_initial_config(config.MODULE_PATH, write_config=False)
+    _, sandboxPath, READ_ONLY, DEBUG = create_initial_config(config.MODULE_PATH, 
+                                                             config_path=config.CONFIG_DIR,
+                                                             write_config=False)
 #
-    if not os.path.exists(os.path.join(sandboxPath, 'sources.csv')):
-#        os.mkdir(sandboxPath)
-        create_empty_datashelf(sandboxPath)
-#    if database == 'sandbox':
+    if os.path.exists(os.path.join(sandboxPath, 'sources.csv')):
+        shutil.rmtree(sandboxPath)
+    
+    create_empty_datashelf(sandboxPath)
     config.PATH_TO_DATASHELF = sandboxPath
     config.SOURCE_FILE = os.path.join(sandboxPath, 'sources.csv')
     dt.core.DB  = dt.database.Database()
 
     _re_link_functions(dt)
-    _create_test_tables()
\ No newline at end of file
+    _create_test_tables()
+    
+def naming_convention_test():
+    #%%
+    from naming_convention import entities
+    
+    
+    variable_list = dt.findp().loc[:,['variable', 'entity']]
+    entities_to_test = tuple(entities)
+    
+    inconsistent_ids = list()
+    nan_ids          = list()
+    for idx, (variable, entity) in variable_list.iterrows():
+        if pd.isnull(entity):
+            nan_ids.append(idx)
+        else:
+            if not idx.startswith(entities_to_test):
+                inconsistent_ids.append(idx)
+                
+    #%%
+    # finding entity
+    tableList = list()
+    for idx in nan_ids:
+        
+        variable, entity = variable_list.loc[idx,:]
+        if idx.startswith(entities_to_test):
+            candiates = list()
+            
+            for entity in entities_to_test:
+                if idx.startswith(entity):
+                    candiates.append(entity)
+                    
+                sort_candiates = sorted(candiates, key=len)
+            
+            entity = sort_candiates[-1]
+            category = variable.lstrip(entity).lstrip('|')
+            table = dt.getTable(idx)
+            table.meta['category'] = category
+            table.meta['entity'] = entity
+            tableList.append(table)
+    dt.updateTables([x.ID for x in tableList], tableList, 'added entity')
