@@ -8,10 +8,14 @@ in various locations and tools.
 """
 
 import time
-from datatoolbox import config
-import numpy as np
-tt = time.time()
 import pint
+
+import numpy as np
+import pandas as pd
+from datatoolbox import config
+
+tt = time.time()
+
 #%% Pint unit handling
 gases = {"CO2eq":"carbon_dioxide_equivalent",
          "CO2e" : "CO2eq",
@@ -42,6 +46,11 @@ if config.AVAILABLE_XARRAY:
     from .tools import xarray as _xr
     to_XDataSet = _xr.to_XDataSet
     to_XDataArray = _xr.to_XDataArray
+    
+    import pint_xarray
+    pint_xarray.accessors.setup_registry(ur)
+    pint_xarray.unit_registry = ur
+    pint_xarray.accessors.default_registry = ur
 
 #%%
 
@@ -80,12 +89,13 @@ def _update_meta(metaDict):
     
     for key in list(metaDict.keys()):
         if (metaDict[key] is np.nan) or metaDict[key] == '':
-            del metaDict[key]
+            if key != 'unit':
+                del metaDict[key]
             
     for id_field in config.ID_FIELDS:
         fieldList = [ metaDict[key] for key in config.SUB_FIELDS[id_field] if key in  metaDict.keys()]
         if len(fieldList)>0:
-            metaDict[id_field] =  config.SUB_SEP[id_field].join(fieldList).strip('|')
+            metaDict[id_field] =  config.SUB_SEP[id_field].join([str(x) for x in fieldList]).strip('|')
     
     return metaDict
 
@@ -94,6 +104,48 @@ def _createDatabaseID(metaDict):
     
     return config.ID_SEPARATOR.join([metaDict[key] for key in config.ID_FIELDS])
 
+def csv_writer(filename, 
+               dataframe,
+               meta,
+               index=0):
+    fid = open(filename,'w', encoding='utf-8')
+    fid.write(config.META_DECLARATION)
+    
+    for key, value in sorted(meta.items()):
+#            if key == 'unit':
+#                value = str(value.u)
+        fid.write(key + ',' + str(value) + '\n')
+    
+    fid.write(config.DATA_DECLARATION)
+    if index == 0:
+        dataframe.to_csv(fid, sep = ',')
+    elif index is None:
+        dataframe.to_csv(fid,index=None,sep = ';')
+    fid.close()
+
+def excel_writer(writer,
+                 dataframe,
+                 meta,
+                 sheet_name="Sheet1",
+                 index=False,
+                 engine=None):
+    
+    if isinstance(writer, pd.ExcelWriter):
+        need_save = False
+    else:
+        writer = pd.ExcelWriter(pd.io.common.stringify_path(writer),  engine=engine)
+        need_save = True
+        
+    
+    metaSeries= pd.Series(data=[''] + list(meta.values()) + [''],
+                              index=['###META###'] + list(meta.keys()) + ['###DATA###'])
+        
+    metaSeries.to_excel(writer, sheet_name=sheet_name, header=None, columns=None)
+    pd.DataFrame(dataframe).to_excel(writer, sheet_name= sheet_name, index= index, startrow=len(metaSeries))
+    
+    if need_save:
+        writer.save()
+            
 
 def osIsWindows():
     """
@@ -238,17 +290,18 @@ def _AR4_conversionFactor(unitFrom, unitTo):
     return factor
 
 
+#%%
 def get_dimension_extend(table_iterable, dimensions):
     """
     This functions assesses the the unique extend for various dimensions
     given a set of datatables
     """
     fullIdx = dict()
-    for dim in dimensions:
-        fullIdx[dim] = set()
+    # for dim in dimensions:
+    #     fullIdx[dim] = set()
 
     for table in table_iterable:
-        
+    
 #        for metaKey, metaValue in table.meta.items():
 #            if metaKey not in metaDict.keys():
 #                metaDict[metaKey] = set([metaValue])
@@ -256,6 +309,10 @@ def get_dimension_extend(table_iterable, dimensions):
 #                metaDict[metaKey].add(metaValue)
             
         for dim in dimensions:
+            
+            if dim not in fullIdx.keys():
+                fullIdx[dim] = set()
+                
             if dim == 'region':
                 fullIdx[dim] = fullIdx[dim].union(table.index)
             elif dim == 'time':
