@@ -14,14 +14,13 @@ import pandas as pd
 import matplotlib.pylab as plt
 import numpy as np
 from copy import copy
-
+import ast
 from . import core
 from . import config 
-#from . import mapping as mapp
+import traceback
 from . import util
-#from . import io_tools
-#from .tools import xarray
-            
+
+       
 class Datatable(pd.DataFrame):
     """
     Datatable
@@ -33,15 +32,29 @@ class Datatable(pd.DataFrame):
 
     def __init__(self, *args, **kwargs):
         
-        metaData = kwargs.pop('meta', {x:'' for x in config.REQUIRED_META_FIELDS})
-
+        # pop meta out of kwargs since pandas is not expecting it
+        overwrite_meta = kwargs.pop('meta', {}) # retrun empty dict if no meta is provide
+        
+        
+        
         super(Datatable, self).__init__(*args, **kwargs)
-#        print(metaData)
-        if metaData['unit'] is None or pd.isna(metaData['unit']):
-            metaData['unit'] = ''
-#        metaData['variable'] = metaData['entity'] + metaData['category']
-        self.__appendMetaData__(metaData)
-        self.vis = Visualization(self)
+        
+        if len(args) > 0 and hasattr(args[0], 'meta'):
+            # print(args)
+            meta = args[0].meta
+            
+        elif 'data' in kwargs.keys() and hasattr(kwargs['data'], 'meta'):
+            meta = kwargs['data'].meta
+            
+        else:
+            meta = {}
+        
+        meta.update(overwrite_meta)
+
+        # append metaDict to datatable
+        self.__appendMetaData__(meta)
+        
+        # self.vis = Visualization(self)
         try:
             self.generateTableID()
         except:
@@ -53,7 +66,33 @@ class Datatable(pd.DataFrame):
             
         self.columns.name = 'time'
         self.index.name   = 'region'
+        
+        # print(self.meta)
+        if ('timeformat' in self.meta.keys()) and (self.meta['timeformat'] != '%Y'):
+            self.columns_to_datetime()
+
+        self.attrs = self.meta
     
+    def info(self):
+        """
+        Returns information about the dataframe like shape, index and column 
+        extend and the number of non-nan entries.
+        
+        Returns
+        -------
+        str
+            Information about datatable.
+
+        """
+        shp = self.shape
+        if (len(self.index) == 0) or (len(self.columns) == 0):
+            return ('Empty Datatable')
+        idx_ext = self.index[0] ,self.index[-1]
+        time_ext = self.columns[0] ,self.columns[-1]
+        n_entries = (~self.isnull()).sum().sum()
+        return f'{shp[0]}x{shp[1]} Datatable with {n_entries} entries, index from {idx_ext[0]} to {idx_ext[1]} and time from {time_ext[0]} to {time_ext[0]}'
+        
+        
     @classmethod
     def from_pyam(cls, idf, **kwargs):
         """
@@ -79,9 +118,9 @@ class Datatable(pd.DataFrame):
         if kwargs:
             idf = idf.filter(**kwargs)
 
-        assert len(idf.variables()) == 1, (
+        assert len(idf.variable) == 1, (
             f"Datatables cannot represent more than one variable, "
-            f"but there are {', '.join(idf.variables())}"
+            f"but there are {', '.join(idf.variable)}"
         )
 
         def extract_unique_values(df, fields, ignore):
@@ -136,7 +175,10 @@ class Datatable(pd.DataFrame):
        
     def _to_xarray(self):
         
-        return core.xr.DataArray(self.values, coords=[self.index, self.columns], dims=['space','time'], attrs=self.meta)
+        return core.xr.DataArray(self.values, 
+                                 coords=[self.index, self.columns], 
+                                 dims=['space','time'], 
+                                 attrs=self.meta)
     
 
     
@@ -172,12 +214,24 @@ class Datatable(pd.DataFrame):
 
 
         """
+        
+        for metaKey in config.REQUIRED_META_FIELDS:
+            if metaKey not in metaDict.keys() or metaDict[metaKey] == '' or pd.isna(metaDict[metaKey]):
+                
+                # Overwrite with default or empty string
+                if metaKey in config.META_DEFAULTS:
+                    metaDict[metaKey] = config.META_DEFAULTS[metaKey]
+                else:
+                    metaDict[metaKey] = ''
+
+        
         self.__setattr__('meta', metaDict.copy())
+        
+        assert core.getUnit(self.meta['unit'])
 
-        #test if unit is recognized
-        #print(self.meta['unit'])
-        core.getUnit(self.meta['unit'])
-
+    def columns_to_datetime(self):
+        self.columns = pd.to_datetime(self.columns, 
+                                      format=self.meta['timeformat'])
 
     def copy(self, deep=True):
         """
@@ -221,6 +275,48 @@ class Datatable(pd.DataFrame):
         
         return out
     
+    def reduce(self, 
+               method='linear_piece_wise',
+               eps = 1e-6):
+        """ 
+        Reduce data that is piecewise linear to the core data points (kinks).
+        """
+        
+        # assert monotonic incease of decrease
+        assert self.columns.is_monotonic
+        
+        #initial value of last year (set to first year)
+        last_year = self.columns[0]
+        last_yearly_change    = np.nan # initial value
+        
+        reduced_data = self.copy()*np.nan
+        
+        for year in self.columns[1:]:
+            #assert year == last_year+1
+            n_years = year - last_year
+            
+            change= self.loc[:,year] - self.loc[:,last_year]
+            
+            # find where is a kink
+            idx = (change - last_yearly_change*n_years).abs() > eps
+            
+            reduced_data.loc[idx, last_year] = self.loc[idx,last_year]
+            
+            # overwrite last values
+            last_yearly_change = change / n_years
+            last_year = year
+        
+        # set first and last values
+        for coISO in self.index:
+            value_columns = self.columns[self.loc[coISO,:].notna()]
+            idx_min = value_columns.min()
+            idx_max = value_columns.max()
+            reduced_data.loc[coISO, idx_min] = self.loc[coISO,idx_min]
+            reduced_data.loc[coISO, idx_max] = self.loc[coISO,idx_max]
+            
+            #%%
+        return reduced_data
+    
     def to_excel(self, fileName = None, sheetName = "Sheet0", writer = None, append=False):
         """
         Save datatable to excel.
@@ -264,7 +360,7 @@ class Datatable(pd.DataFrame):
             writer.close()
         
         
-                
+        
     
     def to_csv(self, fileName=None):
         """
@@ -285,17 +381,21 @@ class Datatable(pd.DataFrame):
         else:
             assert fileName[-4:]  == '.csv'
         
-        fid = open(fileName,'w', encoding='utf-8')
-        fid.write(config.META_DECLARATION)
+#         fid = open(fileName,'w', encoding='utf-8')
+#         fid.write(config.META_DECLARATION)
         
-        for key, value in sorted(self.meta.items()):
-#            if key == 'unit':
-#                value = str(value.u)
-            fid.write(key + ',' + str(value) + '\n')
+#         for key, value in sorted(self.meta.items()):
+# #            if key == 'unit':
+# #                value = str(value.u)
+#             fid.write(key + ',' + str(value) + '\n')
         
-        fid.write(config.DATA_DECLARATION)
-        super(Datatable, self).to_csv(fid)
-        fid.close()
+#         fid.write(config.DATA_DECLARATION)
+#         super(Datatable, self).to_csv(fid)
+#         fid.close()
+        
+        core.csv_writer(fileName,
+                        pd.DataFrame(self),
+                        self.meta)
 
     def to_pyam(self, **kwargs):
         """
@@ -409,7 +509,7 @@ class Datatable(pd.DataFrame):
         Returns
         -------
         datatable
-            Interpoltated dataframe.
+            Interpolated dataframe.
 
         """
         from datatoolbox.tools.for_datatables import interpolate
@@ -419,6 +519,7 @@ class Datatable(pd.DataFrame):
                 if col not in self.columns:
                     self.loc[:,col] = np.nan
             self = self.loc[:,list(range(self.columns.min(),self.columns.max()+1))]
+        
         return interpolate(self, method)
     
     def clean(self):
@@ -508,7 +609,8 @@ class Datatable(pd.DataFrame):
         self.meta['ID'] = self.ID
         return self.ID
 
-
+    def _update_meta(self):
+        self.meta =  core._update_meta(self.meta)
     
     def source(self):
         """
@@ -676,7 +778,7 @@ class Datatable(pd.DataFrame):
             out.values[:] *= newUnit.m
         else:
             out = Datatable(super(Datatable, self).__rtruediv__(other))
-            out.meta['unit'] = (core.getUnit(self.meta['unit'])**-1).u
+            out.meta['unit'] = str((core.getUnit(self.meta['unit'])**-1).u)
             out.meta['source'] = 'calculation'
         return out
     
@@ -692,18 +794,6 @@ class Datatable(pd.DataFrame):
         outStr += super(Datatable, self).__repr__()
         return outStr
     
-    def __str__(self):
-        outStr = """"""
-        if 'ID' in self.meta.keys():
-            outStr += '=== Datatable - ' + self.meta['ID'] + ' ===\n'
-        else:
-            outStr += '=== Datatable ===\n'
-        for key in self.meta.keys():
-            if self.meta[key] is not None:
-                outStr += key + ': ' + str(self.meta[key]) + ' \n'
-        outStr += super(Datatable, self).__str__()
-        return outStr
-    
     def _repr_html_(self):
         outStr = """"""
         if 'ID' in self.meta.keys():
@@ -742,127 +832,38 @@ class TableSet(dict):
         if IDList is not None:
             for tableID in IDList:
                 self.add(core.DB.getTable(tableID))
+    
+    def __getitem__(self, key):
+        item = super(TableSet, self).__getitem__(key)
+        
+        #load datatable if necessary
+        if item is None:
+            item = core.DB.getTable(key)
+            self[key] = item
+        
+        return item
+
+    def __setitem__(self, key, datatable):
+        super(TableSet, self).__setitem__(key, datatable)
+        
+        if datatable.ID is None:
+            try:
+                datatable.generateTableID()
+            except:
+#                print('Could not generate ID, key used instead')
+                datatable.ID = key
+        self.inventory.loc[datatable.ID, "key"] = key
+        self.inventory.loc[datatable.ID, config.INVENTORY_FIELDS] = [datatable.meta.get(x,None) for x in config.INVENTORY_FIELDS]
+    
         
         
-    @classmethod
-    def from_list(cls,
-                  tableList):
-        """
-        Create tableSet form list of datatables.
 
-        Parameters
-        ----------
-        cls : TYPE
-            DESCRIPTION.
-        tableList : list
-            List of datatables.
-
-        Returns
-        -------
-        tableSet : tableset
-            DESCRIPTION.
-
-        """
-        tableSet = cls()
-        for table in tableList:
-            tableSet.add(table)
     
-        return tableSet
-    
-            
-    def to_xarray(self, dimensions):
-        """
-        Convert tableset to and xarray with the given dimenions.
-        Requires xarray installed
 
-        Parameters
-        ----------
-        dimensions : list of str
-            List of xarray dimensions.
-
-        Returns
-        -------
-        xr.xarray
-            DESCRIPTION.
-
-        """
-        if not config.AVAILABLE_XARRAY:
-            raise(BaseException('module xarray not available'))
-        return core.to_XDataArray(self, dimensions)
-       
-    def to_xset(self, dimensions = ['region', 'time']):
-        """
-        Convert table set to an xarray data set.
-
-        Parameters
-        ----------
-        dimensions : list, optional
-            DESCRIPTION. The default is ['region', 'time'].
-
-        Returns
-        -------
-        xr.Dataset
-            DESCRIPTION.
-
-        """
-        dimensions = ['region', 'time']
-        if not config.AVAILABLE_XARRAY:
-            raise(BaseException('module xarray not available'))
-        return core.to_XDataSet(self, dimensions)
-    
-    def to_list(self):
-        """
-        Convert to list of tables
-
-        Returns
-        -------
-        list
-            List of datatables.
-
-        """
-        return [ self[key] for key in self.keys()]
-    
     def __iter__(self):
         return iter(self.values())
     
-    def add(self, datatables=None, tableID=None):
-        """
-        Add new tables to table set. Either datatables or table IDs should be given.
 
-        Parameters
-        ----------
-        datatables : list of datatables, optional
-            DESCRIPTION. The default is None.
-        tableID : TYPE, optional
-            DESCRIPTION. The default is None.
-
-        Returns
-        -------
-        None.
-
-        """
-        
-        # assert only on parameter is None
-        assert not ((datatables is None) and (tableID is None))
-        
-        if datatables is not None:
-            
-            if isinstance(datatables, list):
-                self._add_list(datatables)
-                
-            elif isinstance(datatables, TableSet):
-                self._add_TableSet(datatables)
-                
-            elif isinstance(datatables, Datatable):
-                self._add_single_table(datatables)
-                
-            else:
-                 print('Data type not recognized.')
-            
-        elif tableID is not None:
-            self._add_tableID(tableID)
-            
-            
             
     def _add_list(self, tableList):
         for table in tableList:
@@ -899,12 +900,77 @@ class TableSet(dict):
         # make sure the data is compatible
 #        print(table.meta)
 #        print(self[tableKey].meta)
-        assert table.meta == self[tableKey].meta
+        if table.meta != self[tableKey].meta:
+            raise(BaseException('Trying to update table with different meta'))
         
         # update data
         self[tableKey] = pd.concat([self[tableKey] , table])
             
+    def add(self, datatables=None, tableID=None):
+        """
+        Add new tables to table set. Either datatables or table IDs should be given.
 
+        Parameters
+        ----------
+        datatables : list of datatables, optional
+            DESCRIPTION. The default is None.
+        tableID : TYPE, optional
+            DESCRIPTION. The default is None.
+
+        Returns
+        -------
+        None.
+
+        """
+        
+        # assert only on parameter is None
+        assert not ((datatables is None) and (tableID is None))
+        
+        if datatables is not None:
+            
+            if isinstance(datatables, list):
+                self._add_list(datatables)
+                
+            elif isinstance(datatables, TableSet):
+                self._add_TableSet(datatables)
+                
+            elif isinstance(datatables, Datatable):
+                self._add_single_table(datatables)
+                
+            else:
+                 print('Data type not recognized.')
+            
+        elif tableID is not None:
+            self._add_tableID(tableID)
+          
+    
+    def convert(self, newUnit):
+        """
+        Convert all tables to a new Unit. Returns a copy of the old tableSet
+
+        Parameters
+        ----------
+        newUnit : TYPE
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        """
+        tables = self.copy()
+        
+        for tableKey in tables.keys():
+            
+            tables[tableKey].convert(newUnit)
+            tables[tableKey].meta['unit'] = newUnit
+            tables.inventory.loc[tableKey,'unit'] = newUnit
+            
+        return tables
+    
+    def copy(self):
+        return copy(self)
+    
     def remove(self, tableID):
         """
         Remove table form tableSet.
@@ -922,6 +988,44 @@ class TableSet(dict):
         del self[tableID]
         self.inventory.drop(tableID, inplace=True)
         
+    def filterp(self, level=None, regex=False, **filters):
+        """ 
+        Future defaulf find method that allows for more
+        sophisticated syntax in the filtering
+        
+        Usage:
+        -------
+        filters : Union[str, Iterable[str]]
+            One or multiple patterns, which are OR'd together
+        regex : bool, optional
+            Accept plain regex syntax instead of shell-style, default: False
+        
+        Returns
+        -------
+        matches : pd.Series
+        Mask for selecting matched rows
+        """    
+            
+        # filter by columns and list of values
+        keep = True
+
+        for field, pattern in filters.items():
+            # treat `col=None` as no filter applied
+            if pattern is None:
+                continue
+
+            if field not in self.inventory:
+                raise ValueError(f'filter by `{field}` not supported')
+
+            keep &= util.pattern_match(
+                self.inventory[field], pattern, regex=regex
+            )
+
+        if level is not None:
+            keep &= self.inventory['variable'].str.count(r"\|") == level
+
+        return self.inventory if keep is True else self.inventory.loc[keep]
+            
     
     def filter(self,**kwargs):
         """
@@ -953,6 +1057,31 @@ class TableSet(dict):
             
         return newTableSet
 
+    @classmethod
+    def from_list(cls,
+                  tableList):
+        """
+        Create tableSet form list of datatables.
+
+        Parameters
+        ----------
+        cls : TYPE
+            DESCRIPTION.
+        tableList : list
+            List of datatables.
+
+        Returns
+        -------
+        tableSet : tableset
+            DESCRIPTION.
+
+        """
+        tableSet = cls()
+        for table in tableList:
+            tableSet.add(table)
+    
+        return tableSet
+    
     def aggregate_to_region(self, mapping):
         """ 
         This functions added the aggregates to the output according to the provided
@@ -963,33 +1092,49 @@ class TableSet(dict):
         return util.aggregate_tableset_to_region(self, mapping)
         
         
-    def __getitem__(self, key):
-        item = super(TableSet, self).__getitem__(key)
-        
-        #load datatable if necessary
-        if item is None:
-            item = core.DB.getTable(key)
-            self[key] = item
-        
-        return item
-
-    def __setitem__(self, key, datatable):
-        super(TableSet, self).__setitem__(key, datatable)
-        
-        if datatable.ID is None:
-            try:
-                datatable.generateTableID()
-            except:
-#                print('Could not generate ID, key used instead')
-                datatable.ID = key
-        self.inventory.loc[datatable.ID, "key"] = key
-        self.inventory.loc[datatable.ID, config.INVENTORY_FIELDS] = [datatable.meta.get(x,None) for x in config.INVENTORY_FIELDS]
     
+        #%%
+        
+    def to_compact_excel(self,
+                         writer,
+                         sheet_name="Sheet1",
+                         include_id=False,
+                         meta_columns = None):
+        
+        use_index = include_id
 
+        if isinstance(writer, pd.ExcelWriter):
+            need_close = False
+        else:
+            writer = pd.ExcelWriter(pd.io.common.stringify_path(writer))
+            need_close = True
+            
+        
+        long, metaDict = self.to_compact_long_format(include_id)
+    
+        if meta_columns is not None:
+            for metaKey in meta_columns:
+                if metaKey not in long.columns:
+                    long.loc[:,metaKey] = metaDict[metaKey]
+            years = util.yearsColumnsOnly(long)
+            long = long.loc[:, meta_columns + years]
+
+    
+        core.excel_writer(writer,
+                         long,
+                         metaDict,
+                         sheet_name=sheet_name,
+                         index= use_index,
+                         engine='xlsxwriter')
+        
+        if need_close:
+            writer.close()
+            
     
     def to_excel(self, 
                  fileName, 
-                 append=False):
+                 append=False,
+                 compact = False):
         """
         Sace TableSet as excel file with individual datatables in individual sheets.
     
@@ -1017,6 +1162,8 @@ class TableSet(dict):
                                     datetime_format='mmm d yyyy hh:mm:ss',
                                     date_format='mmmm dd yyyy')  
         
+
+            
         for i,eKey in enumerate(self.keys()):
             table = self[eKey].dropna(how='all', axis=1).dropna(how='all', axis=0)
             sheetName = str(i) + table.meta['ID'][:25]
@@ -1097,22 +1244,154 @@ class TableSet(dict):
         resultTable.meta.update(new_meta)
             
         return resultTable
+
+    def get_all_meta_keys(self):
+        meta_columns = set()
+        for key in self.keys():
+            meta = self[key].meta
+            
+            meta_columns = meta_columns.union(meta.keys())
+        return list(meta_columns)
+    
+    
+    def to_compact_long_format(self, 
+                                include_id= False):
         
-    def to_LongTable(self):
+        meta_columns = ['region'] + self.get_all_meta_keys()
+        # meta_columns = ['region'] + config.INVENTORY_FIELDS
+        if include_id:
+            meta_columns.append('ID')
+        
+                
+        
+        long = self.to_LongTable(meta_list = meta_columns)
+        
+        single_meta = dict()
+        multi_meta  = list()
+        columns_to_drop = list()
+        
+        for column in meta_columns:
+            unique_entries = long.loc[:,column].unique()
+            if len(unique_entries) == 1:
+                if unique_entries[0] != '':
+                    single_meta[column] = unique_entries[0]
+                
+                columns_to_drop.append(column)
+            else:
+                multi_meta.append(column)
+        long = long.drop(columns_to_drop, axis = 1)
+        
+        metaDict = dict()
+        metaDict.update(single_meta)
+        metaDict.update({'meta_columns' : multi_meta})
+        
+        if include_id:
+            long = long.set_index('ID')
+        
+        return long, metaDict
+
+    def to_csv(self,
+               filename):
+        """
+        Conversion to compact lone csv format
+
+        Parameters
+        ----------
+        filename : TYPE
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        """
+        #%%
+
+        long, metaDict = self._compact_to_long_format()
+        
+        core.csv_writer(filename, 
+                        long,
+                        metaDict,
+                        index=None)
+    #%%           
+    def to_xarray(self, dimensions=None):
+        """
+        Convert tableset to and xarray with the given dimenions.
+        Requires xarray installed
+
+        Parameters
+        ----------
+        dimensions : list of str
+            List of xarray dimensions.
+
+        Returns
+        -------
+        xr.xarray
+            DESCRIPTION.
+
+        """
+        if dimensions is None:
+            dimensions = ['region', 'time']
+            for col in config.ID_FIELDS + ['unit']:
+                if len(self.inventory.loc[:,col].unique()) > 1:
+                    dimensions.append(col)
+                    
+        if "unit" in dimensions:
+            raise(BaseException('Different units in dataset can not be merged in one xarray'))
+        
+        if not config.AVAILABLE_XARRAY:
+            raise(BaseException('module xarray not available'))
+        return core.to_XDataArray(self, dimensions)
+       
+    def to_xset(self, dimensions = ['region', 'time']):
+        """
+        Convert table set to an xarray data set.
+
+        Parameters
+        ----------
+        dimensions : list, optional
+            DESCRIPTION. The default is ['region', 'time'].
+
+        Returns
+        -------
+        xr.Dataset
+            DESCRIPTION.
+
+        """
+        dimensions = ['region', 'time']
+        if not config.AVAILABLE_XARRAY:
+            raise(BaseException('module xarray not available'))
+        return core.to_XDataSet(self, dimensions)
+    
+    def to_list(self):
+        """
+        Convert to list of tables
+
+        Returns
+        -------
+        list
+            List of datatables.
+
+        """
+        return [ self[key] for key in self.keys()]
+    
+        
+    def to_LongTable(self, meta_list = ['variable', 'region','scenario', 'model', 'unit']):
         tables = []
 
         for variable, df in self.items():
             if df.empty:
                 continue
-            
+            inp_dict = dict()    
+            for metaKey in meta_list:
+                if metaKey == 'region':
+                    inp_dict[metaKey] = df.index
+                else:
+                    inp_dict[metaKey] = df.meta.get(metaKey, '')
+                
             try:
-                df = df.assign(
-                    region=df.index,
-                    variable=df.meta['variable'],
-                    unit=df.meta['unit'],
-                    scenario=df.meta["scenario"],
-                    model=df.meta.get("model", "")
-                ).reset_index(drop=True)
+                df = pd.DataFrame(df.assign(**inp_dict)).reset_index(drop=True)
+                
             except KeyError as exc:
                 raise AssertionError(f"meta of {variable} does not contain {exc.args[0]}")
  
@@ -1121,8 +1400,8 @@ class TableSet(dict):
         long_df = pd.concat(tables, ignore_index=True, sort=False)
         
         # move id columns to the front
-        id_cols = pd.Index(['variable', 'region', 'scenario', 'model', 'unit'])
-        long_df = long_df[id_cols.union(long_df.columns)]
+        id_cols = pd.Index(meta_list)
+        long_df = long_df[list(id_cols) + list( long_df.columns.difference(id_cols))]
         long_df = pd.DataFrame(long_df)
         return long_df
 
@@ -1181,7 +1460,6 @@ class TableSet(dict):
         plt.yticks([x +.5 for x in range(len(avail.index))], avail.index)
         plt.xticks([x +.5 for x in range(len(avail.columns))], avail.columns, rotation=45)
 
-
 class Visualization():
     """ 
     This class addes handy built-in visualizations to datatables
@@ -1190,14 +1468,21 @@ class Visualization():
     def __init__(self, df):
         self.df = df
     
-    def availability(self):
-        data = np.isnan(self.df.values)
-        availableRegions = self.df.index[~np.isnan(self.df.values).any(axis=1)]
-        print(availableRegions)
+    def availability(self, regions = None):
+        
+        if regions is not None:
+            available_regions = self.df.index.intersection(regions)
+            
+            data = self.df.loc[available_regions, :]
+        else:
+            data = self.df
+            
+            availableRegions = data.index[~data.isnull().all(axis=1)]
+        #print(availableRegions)
         plt.pcolormesh(data, cmap ='RdYlGn_r')
         self._formatTimeCol()
-        self._formatSpaceCol()
-        return availableRegions
+        self._formatSpaceCol(data.index)
+        return available_regions
         
     def _formatTimeCol(self):
         years = self.df.columns.values
@@ -1208,8 +1493,11 @@ class Visualization():
         plt.xticks(xTickts+.5, years[xTickts], rotation=45)
         print(xTickts)
         
-    def _formatSpaceCol(self):
-        locations = self.df.index.values
+    def _formatSpaceCol(self, regions = None):
+        if regions is None:
+            locations = self.df.index.values
+        else:
+            locations = np.asarray(list(regions))
         
         #dt = int(len(locations) / 10)+1
         dt = 1    
@@ -1384,7 +1672,16 @@ class Visualization():
         else:
             with open(fileName, 'w') as f:
                 f.write(html)
-                
+
+def _try_number_format(x):  
+    try:      
+        return int(x)     
+    except:
+        try:
+            return float(x)
+        except:
+            return x
+            
 def read_csv(fileName):
     
     fid = open(fileName,'r', encoding='utf-8')
@@ -1399,16 +1696,23 @@ def read_csv(fileName):
         if line == config.DATA_DECLARATION:
             break
         dataTuple = line.replace('\n','').split(',')
-        meta[dataTuple[0]] = dataTuple[1].strip()
+        meta[dataTuple[0]] = _try_number_format(dataTuple[1].strip())
         if "unit" not in meta.keys():
             meta["unit"] = ""
     df = Datatable(pd.read_csv(fid, index_col=0), meta=meta)
-    df.columns = df.columns.map(int)
+    
+    if ('timeformat' in df.meta.keys()) and (df.meta['timeformat'] != '%Y'):
+        df.columns_to_datetime()
+    else:
+        df.columns = df.columns.astype(int)
 
     fid.close()
     return df
 
-def read_excel(fileName, sheetNames = None):
+def read_excel(fileName, 
+               sheetNames = None,
+               use_sheet_name_as_keys=False, 
+               force_tableSet = False):
  
     
     if sheetNames is None:
@@ -1416,7 +1720,7 @@ def read_excel(fileName, sheetNames = None):
         sheetNames = xlFile.sheet_names
         xlFile.close()
 
-    if len(sheetNames) > 1:
+    if len(sheetNames) > 1 or force_tableSet:
         out = TableSet()
         for sheet in sheetNames:
             fileContent = pd.read_excel(fileName, sheet_name=sheet, header=None)
@@ -1433,10 +1737,19 @@ def read_excel(fileName, sheetNames = None):
                                       index   = fileContent.loc[columnIdx+1:, 0], 
                                       columns = [int(x) for x in fileContent.loc[columnIdx, 1:]], 
                                       meta    = metaDict)
-                dataTable.generateTableID()
-                out.add(dataTable)
-            except:
-                print('Failed to read the sheet: {}'.format(sheet))
+                
+                if use_sheet_name_as_keys:
+                    out[sheet] = dataTable
+                else:
+                    dataTable.generateTableID()
+                    out.add(dataTable)
+                
+            except Exception:
+
+                
+                if config.DEBUG:
+                    print(traceback.format_exc())
+                print('Failed to read the sheet: {}'.format(sheet))                    
         
     else:
         sheet = sheetNames[0]
@@ -1452,9 +1765,11 @@ def read_excel(fileName, sheetNames = None):
             columnIdx = idx +1
             dataTable = Datatable(data    = fileContent.loc[columnIdx+1:, 1:].astype(float).values, 
                                   index   = fileContent.loc[columnIdx+1:, 0], 
-                                  columns = [int(x) for x in fileContent.loc[columnIdx, 1:]], 
+                                  columns = fileContent.loc[columnIdx, 1:], 
                                   meta    = metaDict)
-            
+            if ('timeformat' in dataTable.meta.keys()) and (dataTable.meta['timeformat'] != '%Y'):
+                dataTable.columns_to_datetime()
+        
             try:
                 dataTable.generateTableID()
             except:
@@ -1464,6 +1779,101 @@ def read_excel(fileName, sheetNames = None):
 #                print('Failed to read the sheet: {}'.format(sheet))
         
     return out
+
+def read_compact_excel(file_name, sheet_name=None):
+    #%%
+    if sheet_name is None:
+        xlFile = pd.ExcelFile(file_name)
+        sheet_names = xlFile.sheet_names
+        xlFile.close()
+
+    
+    out = TableSet()
+    for sheet in sheet_names:
+        fileContent = pd.read_excel(file_name, sheet_name=sheet, header=None)
+        metaDict = dict()
+        
+        if fileContent.loc[0, 0] != '###META###':
+            raise(BaseException('Undefined format'))
+            
+        # try:
+        if True:
+            for idx in fileContent.index[1:]:
+                key, value = fileContent.loc[idx, [0,1]]
+                if key == '###DATA###':
+                    break
+                
+                metaDict[key] = value
+            columnIdx = idx +1
+            
+
+            lDf = fileContent.loc[columnIdx+1:, :]
+            lDf.columns = fileContent.loc[columnIdx, :]
+            # lDf.index = fileContent.loc[columnIdx+1:, 0]
+            
+            if 'ID' in metaDict['meta_columns']:
+                lDf.loc[:,'ID'] = fileContent.loc[columnIdx:, 0]
+            # import json
+            # x = '[ "A","B","C" , " D"]'
+            # json.loads(metaDict['meta_columns'])
+            
+            try:
+                meta_columns = ast.literal_eval(metaDict['meta_columns'])
+            except:
+                meta_columns = metaDict['meta_columns']
+            if isinstance(meta_columns, str):
+                meta_columns= meta_columns.split(', ')
+            # if 'region' in meta_columns:
+            meta_columns.remove('region')
+                
+            for idx, line in lDf.iterrows():
+                
+                #print(idx)
+                    
+                meta = { x: metaDict[x] for x in metaDict.keys() if x not in  ['meta_columns']}
+                
+                for meta_col in meta_columns:
+                    meta[meta_col] = line[meta_col]
+                region = line.loc['region']
+                
+                if 'ID' in line.index:
+                    ID = line.loc['ID']
+                else:
+                    try:
+                        meta = core._update_meta(meta)
+                        ID = core._createDatabaseID(meta)
+                    except:
+                        
+                        ID = config.ID_SEPARATOR.join([meta[key] for key in [x for x in config.ID_FIELDS if x in meta.keys()]])
+                        print(f'ID could not properly created, fallback to partial ID: {ID}')
+                # if config.DEBUG:
+                    # print(f'meta columns: {meta_columns}')
+                    # print(f'numerical columns: {list(lDf.columns.difference(meta_columns + ["region"]))}')
+                numerical_columns = list(lDf.columns.difference(meta_columns + ['region']).astype(int))
+                
+                if ID not in out.keys():
+                    # table  = Datatable()
+                    table = Datatable(
+                                columns= numerical_columns,
+                                index= [region],                
+                                meta = meta)
+                    
+                    table.loc[region, numerical_columns] = line.loc[numerical_columns].astype(float)
+                    out[ID] = table
+                    
+                else:
+                    line.loc[numerical_columns].astype(float)
+                    factor = core.conversionFactor(meta['unit'], out[ID].meta['unit'])
+                    out[ID].loc[region, numerical_columns] = line.loc[numerical_columns].astype(float) *factor
+                    #
+    #%%
+                
+    return out
+            
+        # except:
+        #         print('Failed to read the sheet: {}'.format(sheet))
+
+
 #%%
 class MetaData(dict):
     
