@@ -15,17 +15,18 @@ import types
 import traceback
 from collections import defaultdict
 from pathlib import Path
+import shutil
 
 import pandas as pd
 import numpy as np
 
 from . import config
 from .data_structures import Datatable, TableSet, read_csv
-from .utilities import plot_query_as_graph, shorten_find_output
 from . import mapping as mapp
 from . import io_tools as io
 from . import util
 from . import core
+from .tools import xarray
 
 # helper funtions
 def restore_source(repoName):
@@ -52,7 +53,34 @@ def restore_source(repoName):
     repoPath = os.path.join(config.PATH_TO_DATASHELF,  'database', repoName)
     git_repo = git.Repo(repoPath)
     git_repo.git.execute(['git', 'reset', '--hard', git_hash])
-            
+    
+def unique(table, field=None):
+    #%%
+    unique_dict = dict()
+    max_len = 0
+    if field is None:
+        fields = config.ID_FIELDS
+    else:
+        fields = [field]
+    
+    for var in fields:
+        unique_dict[var] = table.loc[:,var].unique()
+        max_len = max(max_len, len(unique_dict[var]))
+    unique_data = pd.DataFrame(index = range(max_len),
+                              columns = fields,
+                              data = '').astype(str)
+    
+    for var in fields:
+        len_data = len(unique_dict[var])
+        unique_data.loc[unique_data.index[:len_data],var] = unique_dict[var]
+    with pd.option_context('display.max_rows', None, 'display.max_columns', None):  # more options can be specified also
+        print(unique_data)
+        
+        
+
+    #%%
+    
+        
 class Database():
     """
     CSV based database that uses git for as distributed version control system.
@@ -77,7 +105,7 @@ class Database():
         self.inventory = pd.read_csv(self.INVENTORY_PATH, index_col=0, dtype={'source_year': str})
         self.sources   = self.gitManager.sources
         
-        self.gitManager._validateRepository('main')
+        #self.gitManager._validateRepository('main')
 
 
         if config.DEBUG:
@@ -207,24 +235,26 @@ class Database():
         self.inventory.drop(tableID, inplace=True)
 #        self.gitManager.updatedRepos.add('main')
     
-    def getInventory(self, **kwargs):
+    def findc(self, **kwargs):
         """
         Method to search through the inventory. kwargs can be all inventory entires
         (see config.INVENTORY_FIELDS).
         """
         
-        table = self.inventory.copy()
         # loop over all keys of kwargs to filter based on all of them
-        for key in kwargs.keys():
-            
-            mask = self.inventory[key].str.contains(kwargs[key], regex=False)
-            mask[pd.isna(mask)] = False
-            mask = mask.astype(bool)
-            table = table.loc[mask].copy()
+        mask = True
+        for key, val in kwargs.items():
+            mask &= self.inventory[key].str.contains(val, regex=False, na=False)
+
+        table = (
+            self.inventory if mask is True else self.inventory.loc[mask]
+        ).copy()
         
         # test to add function to a instance (does not require class)
-        table.graph = types.MethodType( plot_query_as_graph, table )
-        table.short = types.MethodType(shorten_find_output, table)
+        table.graph = types.MethodType(util.plot_query_as_graph, table)
+        table.short = types.MethodType(util.shorten_find_output, table)
+        table.unique = types.MethodType(unique, table)
+        table.to_pyam = types.MethodType(util.to_pyam, table)
         return table
 
     def findp(self, level=None, regex=False, **filters):
@@ -263,13 +293,24 @@ class Database():
         if level is not None:
             keep &= self.inventory['variable'].str.count(r"\|") == level
 
-        table = self.inventory.loc[keep]
+        table = pd.DataFrame(
+            self.inventory if keep is True else self.inventory.loc[keep]
+        )
 
         # test to add function to a instance (does not require class)
-        table.graph = types.MethodType(plot_query_as_graph, table)
-        table.short = types.MethodType(shorten_find_output, table)
+        table.graph   = types.MethodType(util.plot_query_as_graph, table)
+        table.short   = types.MethodType(util.shorten_find_output, table)
+        table.unique  = types.MethodType(unique, table)
+        
+        table.filterp  = types.MethodType(util.filterp, table)
+        table.load    = types.MethodType(self.getTables, table.index) 
+        table.as_pyam = types.MethodType(util.to_pyam, table)
+        table.as_xarray = types.MethodType(xarray.load_as_xdataset, 
+                                           table)
+        table.as_wide_dataframe    = types.MethodType(self.query_to_long_table, table) 
         return table
-
+    
+    
     
     def findExact(self, **kwargs):
         """
@@ -283,8 +324,9 @@ class Database():
             table = table.loc[mask].copy()
         
         # test to add function to a instance (does not require class)
-        table.graph = types.MethodType( plot_query_as_graph, table )
-        table.short = types.MethodType(shorten_find_output, table)
+        table.graph = types.MethodType(util.plot_query_as_graph, table )
+        table.short = types.MethodType(util.shorten_find_output, table)
+        table.to_pyam = types.MethodType(util.to_pyam, table)
         return table
     
     def _getTableFilePath(self,ID):
@@ -303,6 +345,8 @@ class Database():
         """
         return ID.replace('|','-').replace('/','-') + '.csv'
 
+    def _get_source_path(self, sourceID):
+        return  os.path.join(config.PATH_TO_DATASHELF, 'database', sourceID)
     
     def getTable(self, ID):
         """
@@ -338,6 +382,10 @@ class Database():
             # trying to import sources from remote on demand
             source = self._getSourceFromID(ID)
             
+            if source in core.DB.sources.index:
+                
+                raise(BaseException('Table {} not found'.format(ID)))
+            
             try: 
                 if config.DEBUG:
                     print('Trying to import source {}'.format(source))
@@ -430,13 +478,13 @@ class Database():
 
         """
         #create folder if required
-        if ~os.path.exists(folder):
+        if  not os.path.exists(folder):
             os.mkdir(folder)
         #save tables to disk
         self.saveTablesToDisk(folder, core.LOG['tableIDs'])
         if config.DEBUG:
             print('{} tables stored to directory {}'.format(
-                len(core.LOG['tableIDS']), folder))
+                len(core.LOG['tableIDs']), folder))
         
         
     def saveTablesToDisk(self, folder, IDList):
@@ -526,6 +574,8 @@ class Database():
         
         else:
             for dataTable in tqdm.tqdm(dataTables):
+                if config.DEBUG:
+                    print(dataTable.ID)
                 if cleanTables:
                     dataTable = util.cleanDataTable(dataTable)
                 
@@ -552,7 +602,35 @@ class Database():
                 
         self._gitCommit(message)
         
+    def update_mapping_file(self, 
+                            sourceID, 
+                            mapping_file_path,
+                            sourceMetaDict = None):
+        """
+        adds mapping file to database
 
+        Parameters
+        ----------
+        source : TYPE
+            DESCRIPTION.
+        ID : TYPE
+            DESCRIPTION.
+        mapping : TYPE
+            DESCRIPTION.
+            """
+        if not core.DB.isSource(sourceMetaDict['SOURCE_ID']):
+            if sourceMetaDict is None:
+                raise(BaseException('Source meta can not be None'))
+            core.DB._addNewSource(sourceMetaDict)
+
+        source_path = self._get_source_path(sourceID)
+        # for key in mapping.keys():
+        dest_file_path = os.path.join(source_path, 'mapping.xlsx')
+        shutil.copyfile(mapping_file_path, dest_file_path)
+            # mapping[key].to_csv(file_path)
+        
+        self.gitManager.gitAddFile(sourceID, dest_file_path)
+        
     def updateTable(self, oldTableID, newDataTable, message):
         """
         Specific method to update the data of an existing table
@@ -598,6 +676,7 @@ class Database():
             
             if oldTableID in self.inventory.index:
                 self._updateTable(oldTableID, newDataTable)
+                
             else:
                 dataTable = util.cleanDataTable(newDataTable)
                 self._addTable(dataTable)
@@ -628,7 +707,7 @@ class Database():
 
             #change inventory
             self.inventory.rename(index = {oldID: newID}, inplace = True)
-            self.add_to_inventory(newDataTable)
+        self.add_to_inventory(newDataTable)
 
 
     def validate_ID(self, ID, print_statement=True):
@@ -857,14 +936,13 @@ class Database():
         """
         Function to remove an entire source from the database. 
         """
-        import shutil
+        
         if self.sourceExists(sourceID):
             sourcePath = os.path.join(config.PATH_TO_DATASHELF, 'database', sourceID)
             shutil.rmtree(sourcePath, ignore_errors=False, onerror=None)
         self.gitManager.sources = self.gitManager.sources.drop(sourceID, axis=0)
         self.sources            = self.gitManager.sources
-        tableIDs = self.inventory.index[self.inventory.source==sourceID]
-        self.inventory = self.inventory.drop(tableIDs, axis=0)
+        self.inventory = self.inventory.loc[self.inventory.source != sourceID]
 #        self.gitManager.updatedRepos.add('main')
         self._gitCommit(sourceID + ' deleted')
     
@@ -904,8 +982,7 @@ class Database():
         self.gitManager.clone_source_from_remote(remoteName, repoPath)
 
         sourceInventory = pd.read_csv(os.path.join(repoPath, 'source_inventory.csv'), index_col=0, dtype={'source_year': str})
-        for idx in sourceInventory.index:
-            self.inventory.loc[idx,:] = sourceInventory.loc[idx,:]
+        self.inventory = self.inventory.append(sourceInventory, verify_integrity=True)
         self._gitCommit('imported ' + remoteName)
 
     def exportSourceToRemote(self, sourceID):
@@ -939,8 +1016,35 @@ class Database():
         
         self._gitCommit('udpate from remote')
         
+    
+    def query_to_long_table(self, 
+                            query, 
+                            meta_list = ['variable', 'region','scenario', 'model', 'unit']):
+    
+    
+        tables = [self.getTable(x) for x in query.index]
+        final_tables = list()
+        for table in tables:
+            if table.empty:
+                continue
+            inp_dict = dict()    
+            for metaKey in meta_list:
+                if metaKey == 'region':
+                    inp_dict[metaKey] = table.index
+                else:
+                    inp_dict[metaKey] = table.meta.get(metaKey, '')
+                
+            try:
+                final_tables.append(pd.DataFrame(table.assign(**inp_dict)).reset_index(drop=True))
+            except KeyError as exc:
+                raise AssertionError(f"meta of {table.ID} does not contain {exc.args[0]}")
+        long_df = pd.concat(final_tables, ignore_index=True, sort=False)
         
-        
+        # move id columns to the front
+        id_cols = pd.Index(meta_list)
+        long_df = long_df[list(id_cols) + list( long_df.columns.difference(id_cols))]
+        long_df = pd.DataFrame(long_df)
+        return long_df       
 #%%
 class GitRepository_Manager:
     """
@@ -1091,20 +1195,23 @@ class GitRepository_Manager:
         
     def create_remote_repo(self, repoName):
         """
-        Function to create a remove git repoisitoy from an existing local repo
+        Function to create a remote git repository from an existing local repo
         """
         repo = self[repoName]
+
         if 'origin' in repo.remotes:
-            print('remote origin already exists, skip')
-            return 
+            # remote origin has been configured already, but re-push anyway, since this
+            # could have been a connectivity issue
+            origin = repo.remotes.origin
+        else:
+            origin = repo.create_remote("origin", config.DATASHELF_REMOTE + repoName + ".git")
 
-        origin = repo.create_remote("origin", config.DATASHELF_REMOTE + repoName + ".git")
-        origin.push(refspec="master:origin")
+        branch = repo.heads.master
+        origin.push(branch, progress=TqdmProgressPrinter())
+
+        # Update references on remote
         origin.fetch()
-
-        # TODO maybe switch next two commands
-        repo.heads.master.set_tracking_branch(origin.refs.master)
-        origin.push(repo.heads.master)
+        branch.set_tracking_branch(origin.refs.master)
     
     def push_to_remote_datashelf(self, repoName):
         """
@@ -1117,7 +1224,7 @@ class GitRepository_Manager:
         function. TODO
 
         """
-        self[repoName].remote('origin').push(progress=TqdmProgressPrinter())
+        self[repoName].remotes.origin.push(progress=TqdmProgressPrinter())
         
     def clone_source_from_remote(self, repoName, repoPath):
         """
@@ -1133,13 +1240,10 @@ class GitRepository_Manager:
             url = config.DATASHELF_REMOTE + repoName + '.git'
             repo = git.Repo.clone_from(url=url, to_path=repoPath, progress=TqdmProgressPrinter())  
         except:
-            
             print('Failed... Try Cloning source via https:')
             url = config.DATASHELF_REMOTE_HTTPS + repoName + '.git'
             repo = git.Repo.clone_from(url=url, to_path=repoPath, progress=TqdmProgressPrinter())
-            
-            
-                    
+
         self.repositories[repoName] = repo
 
         # Update source file
