@@ -5,16 +5,20 @@ Created on Tue Oct 27 11:05:30 2020
 
 @author: ageiges
 """
-
+from .. import config
 from . import  for_datatables 
-#from . import install_support
 
 from . import pandas
 from . import excel
-from . import xarray
+
 from . import matplotlib
-#from . import pyam
 
-#from . import py_magicc_tools
+#%% optional
+if config.AVAILABLE_XARRAY:
+    from . import xarray
 
+if config.AVAILABLE_DOCX:
+    from . import word
+
+#from . import magicc6
 
