@@ -5,9 +5,13 @@ Created on Wed May  8 11:25:15 2019
 
 @author: Andreas Geiges
 """
+
+import os
+import networkx as nx
+
 import pandas as pd
 import numpy as np
-from typing import Union, Iterable, Optional
+from typing import Union, Iterable
 
 import datatoolbox as dt
 from datatoolbox import mapping as mapp
@@ -16,7 +20,7 @@ from datatoolbox import config
 from datatoolbox import data_structures
 from datatoolbox.greenhouse_gas_database import GreenhouseGasTable 
 import matplotlib.pylab as plt
-import os
+
 import tqdm
 
 import deprecated as dp
@@ -53,12 +57,7 @@ colorGenerator = dp.deprecated(_plt.colorGenerator,
 savefig = dp.deprecated(_plt.savefig,
                                reason="please import from matplotlib toolkid", 
                                version='0.4.0')
-from .tools import pandas as _pd
 
-yearsColumnsOnly = dp.deprecated(_pd.yearsColumnsOnly,
-                               reason="please import from pandas toolkid", 
-                               version='0.4.0')
- 
 #%%
 from .tools import html as _html
 
@@ -170,7 +169,7 @@ def cleanDataTable(dataTable):
         )
         dataTable = dataTable.loc[dataTable.index.difference(invalidSpatialIDs)]
     
-    dataTable = dataTable.dropna(how="all", axis=1).dropna(how="all", axis=0)
+    dataTable = dataTable.dropna(how="all", axis=1).dropna(how="all", axis=0).astype(float)
 
     # clean meta data
     keysToDelete = list()
@@ -186,6 +185,7 @@ def cleanDataTable(dataTable):
     # ensure time colums to be integer
     dataTable.columns = dataTable.columns.astype(int)
     
+    dataTable = dataTable.loc[:, dataTable.columns.sort_values()]
     return dataTable
 
 
@@ -203,21 +203,22 @@ def identifyCountry(string):
             return mapp.countries.index[mask][0]
     except:
         pass
-    if len(string) == 2:
-        mask = string == mapp.countries.codes['alpha2']
+    
+    if len(str(string)) == 2:
+        mask = str(string).upper() == mapp.countries.codes['alpha2']
         if mask.any():
             return mapp.countries.codes.index[mask][0]
     
-    if len(string) == 3:
+    if len(str(string)) == 3:
          
-        if string.upper() in mapp.countries.codes.index:
+        if str(string).upper() in mapp.countries.codes.index:
             return string.upper()
     
     try: 
         coISO = dt.getCountryISO(string)
         return coISO
     except:
-        print('not matching country found')    
+        print(f'not matching country found for {string}')    
         return None
             
 def convertIndexToISO(table, iso_type='alpha3'):
@@ -252,14 +253,17 @@ def convertIndexToISO(table, iso_type='alpha3'):
         table.index = mapp.countries.codes.loc[table.index,'numISO'].astype(int)
     return table
 
-def addCountryNames(table):
+def addCountryNames(table, as_index=False):
     names = list()
     for idx in table.index:
         if idx in mapp.countries.codes.index:
             names.append(mapp.countries.codes.loc[idx,'name'])
         else:
-            names.append('')
-    table.loc[:,'country_name'] = names
+            names.append(idx)
+    if as_index:
+        table.index = names    
+    else:
+        table.loc[:,'country_name'] = names
     return table
 
 
@@ -824,6 +828,374 @@ def _create_sandbox_tables(sourceID, random_seed):
     tables.append(table)
     return tables,source_meta
 
+
+# old utilities
+
+
+# open files
+openers = dict()
+
+operation_systems_supported = ['Linux', 'Darwin']
+#excel
+openers['xlsx'] = dict()
+openers['xlsx']['Linux'] = 'libreoffice'
+openers['xlsx']['Darwin'] = 'open -a "Microsoft Excel"'
+
+openers['docx'] = dict()
+openers['docx']['Linux'] = 'libreoffice'
+openers['docx']['Darwin'] = 'open -a "Microsoft Word"'
+
+
+
+def open_file(path):
+    
+    suffix = path.split('.')[-1]
+    
+    if config.OS not in operation_systems_supported:
+        print('OS not supported. Currently support is restriced to ' + str(operation_systems_supported))
+        return 
+    
+    if suffix not in openers.keys():
+        print('no suiable file opener found ')
+        return
+    
+    os.system(' '.join([openers[suffix][config.OS], 
+                        path]))
+        # elif dt.config.OS == 'Darwin':
+        #     os.system('open -a "Microsoft Excel" ' + self.setup.MAPPING_FILE)
+
+
+
+def shorten_find_output(dataframe):
+    return dataframe.reset_index(drop=True).drop(['scenario', 'model', 'category', 'entity', 'source_year', 'source_name', 'unit'], axis=1)
+
+
+def get_data_trees(**kwargs):
+    findc = core.DB.findc
+    results = findc(**kwargs)
+    return _process_query(results)
+    
+
+def _process_query(results):
+    
+    # Initialize graphs for different data heads
+    heads, graphs = [], {}
+    
+    if len(results.scenario.unique()) > 1:
+        raise ValueError(
+            "Multiple scenarios were detected, ensure that the"
+            + " scenario name/ model/ data source etc. specify a unique scenario"
+        )
+    if len(results.scenario.unique()) == 0:
+        raise ValueError(
+            "Specified kwargs point to an empty list of scenarios. "
+            + "Change kwargs or update your database."
+        )
+    scenario = list(results.scenario.unique())[0]
+
+    # Iterate through data inventory
+    for ix in results.index:
+
+        # Remove scenario and model name
+        ix = ix.split("__")[0]
+        nodes = ix.split("|")
+
+        # If first time occurrence of data head, update graphs
+        if nodes[0] not in heads:
+            heads.append(nodes[0])
+            graph = nx.DiGraph()
+            attr = {"label": nodes[0]}
+            graph.add_node(nodes[0], **attr)
+            graphs.update({nodes[0]: graph})
+
+        # Fetch correct graph/  dict/ list
+        graph = graphs[nodes[0]]
+
+        # Add branches to tree
+        root = nodes[0]
+        for i, name_short in enumerate(nodes[1:]):
+            # Get unique node name
+            name_long = "|".join(nodes[1 : i + 2])
+            # Add node to graph if it does not exist already
+            if name_long not in graph.nodes:
+                # Mark with "*"" if it point to a data table
+                label = name_short
+                # Add a short label for better visualization
+                attr = {"label": label}
+                graph.add_node(name_long, **attr)
+            if i == len(nodes[1:]) - 1:
+                graph.nodes[name_long]["label"] = name_short + "*"
+            # Add edge
+            graph.add_edge(root, name_long)
+            root = name_long
+
+    return graphs, scenario
+
+
+def get_positions(graph, x_offset=1):
+    """Get positions of nodes for horizontally aligned tree visualization
+
+    Args:
+        graph (networkx DiGraph): directed graph which is a tree
+        x_offset (int, optional): Offset between horizontal spacing. Defaults to 1.
+
+    Raises:
+        TypeError: if not networkx DiGraph or tree
+
+    Returns:
+        dict: dictionary, mapping nodes to xy positions
+    """
+
+    # Check if tree
+    if not isinstance(graph, nx.DiGraph):
+        raise TypeError("Has to be a networkx DiGraph")
+
+    if not nx.is_tree(graph):
+        raise TypeError("Has to be a tree")
+
+    # Determine root node
+    root = next(iter(nx.topological_sort(graph)))
+
+    # Determine number of subbranches
+    out_degrees = graph.out_degree()
+
+    def nb_leafs_in_subtree(root):
+        """Recursive function for getting the number of leafs attached to root (root inclusive)
+
+        Args:
+            root (networkx node): root of subtree
+
+        Returns:
+            int: number of leafs in subtree
+        """
+        if out_degrees[root] == 0:
+            nb_children = 1
+        else:
+            nb_children = sum(
+                nb_leafs_in_subtree(child) for child in graph.neighbors(root)
+            )
+
+        return nb_children
+
+    def set_positions(
+        root_, x_spacing={}, depth=0, pos={root: (0, nb_leafs_in_subtree(root) / 2)},
+    ):
+        """Sets positions of nodes in a tree for horizontally aligned tree in a recursive fashion
+
+        Args:
+            root_ (networkx node): root of subtree
+            x_spacing (dict, optional): Dictionary for keeping track of required horizontal spacing. Defaults to {}.
+            depth (int, optional): Current tree depth. Defaults to 0.
+            pos (dict, optional): [description]. Defaults to {root: (0, nb_leafs_in_subtree(root) / 2)}.
+
+        Returns:
+            (dict, dict): Returns  x_spacing and pos
+        """
+
+        # Consider length of root for x-spacing
+        x_spacing.setdefault(depth, len(graph.nodes[root_]["label"]))
+        x_spacing[depth] = max(x_spacing[depth], len(graph.nodes[root_]["label"]))
+
+        if out_degrees[root_] == 0:
+            return
+
+        # Distribute children of root_ across the y-axis
+        offset = 0
+        depth += 1
+        x_spacing.setdefault(depth, 0)
+
+        for child in graph.neighbors(root_):
+            y_pos = (
+                pos[root_][1]
+                - nb_leafs_in_subtree(root_) / 2
+                + nb_leafs_in_subtree(child) / 2
+                + offset
+            )
+            pos.update({child: (depth, y_pos)})
+            offset += nb_leafs_in_subtree(child)
+
+            set_positions(child, x_spacing, depth=depth, pos=pos)
+
+        return pos, x_spacing
+
+    # Determine positions of nodes
+    pos, x_spacing = set_positions(root)
+    # Re-adjust x-spacing
+    pos = {
+        key: (sum(x_spacing[i] + x_offset for i in range(pos_[0])), pos_[1])
+        for key, pos_ in pos.items()
+    }
+
+    return pos, x_spacing
+
+
+def plot_tree(
+    graph,
+    scenario,
+    x_offset=3,
+    fontsize=12,
+    figsize=None,
+    savefig_path=None,
+    dpi=100,
+):
+    """Plots a tree indicating available data of a scenario 
+
+    Parameters
+    ----------
+    graph : networkx.DiGraph
+        tree in digraph format (obtained via get_data_trees function)
+    scenario : str
+        scenario name
+    x_offset : int, optional
+        x offset between nodes, by default 3
+    fontsize : int, optional
+        fontsize of the node labels (either fontsize or figsize can be specified not
+        both), by default 12
+    figsize : 2-dim tuple or None, optional
+        figure size (either fontsize or figsize can be specified not
+        both), by default None
+    savefig_path : str or None, optional
+        path to save figure to (e.g savefig_path = os.path.join(os.getcwd(), "fig.png") ),
+        by default None
+    dpi : int, optional
+        dots per inches used in savefig, by default 100
+    """
+
+    pos, x_spacing = get_positions(graph, x_offset=x_offset)
+
+    if figsize is None:
+        fig, ax = plt.subplots()
+    else:
+        fig, ax = plt.subplots(figsize=figsize)
+
+    # Draw the graph
+    nx.draw(graph, pos=pos, with_labels=False, ax=ax, node_color="none")
+
+    # Set xlim and ylim
+    x_max = (
+        sum(
+            x + x_offset
+            for level, x in x_spacing.items()
+            if type(level) == int
+        )
+        - x_offset
+    )
+    y_max = max(pos_[1] for pos_ in pos.values()) + 1
+    ax.set_xlim([0, x_max]), ax.set_ylim([0, y_max])
+
+    # Get fontsize or reset figsize to avoid overlaps
+    if fontsize is None:
+        x_fig, y_fig = fig.get_size_inches() * fig.dpi
+        fontsize = min(x_fig / (x_max / 1.5), y_fig / (y_max * 2.5))
+    else:
+        x_fig = 2 + (fontsize * (x_max / 1.5) / fig.dpi)
+        y_fig = 2 + (fontsize * y_max * 2.5 / fig.dpi)
+        fig.set_size_inches(x_fig, y_fig, forward=True)
+
+    # Add node labels
+    for node, xy in pos.items():
+        text = graph.nodes[node]["label"]
+        ax.annotate(
+            text,
+            xy,
+            bbox=dict(pad=0.2, fc="gainsboro", ec="k", boxstyle="round"),
+            family="monospace",
+            fontsize=fontsize,
+            verticalalignment="center",
+            horizontalalignment="left",
+        )
+
+    # Add legend
+    ax.annotate(
+        "*: data available\nscenario: {}".format(scenario),
+        (x_max, y_max),
+        family="monospace",
+        fontsize=fontsize,
+        verticalalignment="top",
+        horizontalalignment="right",
+    )
+
+    # Plot and save
+    plt.tight_layout()
+
+    if savefig_path is not None:
+        plt.savefig(savefig_path, dpi=dpi)
+
+    plt.show()
+
+def plot_query_as_graph(results, savefig_path=None):
+    
+   graphs, scenario =  _process_query(results)
+   for gKey in graphs.keys():
+       plot_tree(graphs[gKey], 
+                 scenario, 
+#                 figsize=[5,6],
+                 savefig_path=savefig_path)
+
+def to_pyam(results):
+    return core.DB.getTables(results.index).to_pyam()
+
+def filterp(df, level=None, regex=False, **filters):
+        """ 
+        Future defaulf find method that allows for more
+        sophisticated syntax in the filtering
+        
+        Usage:
+        -------
+        filters : Union[str, Iterable[str]]
+            One or multiple patterns, which are OR'd together
+        regex : bool, optional
+            Accept plain regex syntax instead of shell-style, default: False
+        
+        Returns
+        -------
+        matches : pd.Series
+        Mask for selecting matched rows
+        """    
+            
+        # filter by columns and list of values
+        keep = True
+
+        for field, pattern in filters.items():
+            # treat `col=None` as no filter applied
+            if pattern is None:
+                continue
+
+            if field not in df:
+                raise ValueError(f'filter by `{field}` not supported')
+
+            keep &= pattern_match(
+                df[field], pattern, regex=regex
+            )
+
+        if level is not None:
+            keep &= df['variable'].str.count(r"\|") == level
+
+        return df if keep is True else df.loc[keep]
+def yearsColumnsOnly(index):
+    """
+    Extracts from any given index only the index list that can resemble 
+    as year 
+    
+    e.g. 2001
+    """
+    
+    import re
+    REG_YEAR   = re.compile('^[0-9]{4}$')
+    
+    newColumns = []
+    for col in index:
+        if REG_YEAR.search(str(col)) is not None:
+            newColumns.append(col)
+        else:
+            try: 
+                if ~np.isnan(col) and REG_YEAR.search(str(int(col))) is not None:
+                #   test float string
+                    newColumns.append(col)
+            except:
+                pass
+    return newColumns
+
 #%%    
 if __name__ == '__main__':
     #%%
